#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/19 9:53
# @Author  : wiken
# @Site    : 
# @File    : context_attach_data_server.py
# @Software: PyCharm
# @Desc    : 对mongo数据库执行附加操作
from ys_service.http_service.utils import *


class ContextAttachDataServer(object):

    def __init__(self, handler, message_id=""):
        self.message_id = message_id
        self.attach_data_handler = handler  # 附加数据对象

    def __setitem__(self, key, value):
        """
        设置attach data的属性值
        :param key: 键名
        :param value: 值
        :return:
        """
        props = {key: value}
        data_type = "YS_Hosted"
        data_id = self.message_id

        response = self.attach_data_handler.set(props, data_type=data_type, data_id=data_id)

        print(check_http_response(response))

    def append(self, data):
        """
        向mongo数据库的data_type, data_id 中的attach_list追加数据。
        :param data: 数据, 字典格式{key: value}。
        :return: 操作结果, dict类型。
        """
        return self.attach_data_handler.append(
            data,
            data_type="YS_Hosted",
            data_id=self.message_id
        )

    def get_all(self):
        """
        通过 message_id 查询所有的属性值和attach_list的值。
        :return: 查询结果, dict类型。
        """
        return self.attach_data_handler.get_all(data_type="YS_Hosted", data_id=self.message_id)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/6 15:35
# @Author  : wiken
# @Site    : 
# @File    : timing_server.py
# @Software: PyCharm
# @Desc    :

from apscheduler.schedulers.background import BackgroundScheduler


class TimingService(object):

    cron = "cron"
    interval = "interval"
    date = "date"

    def __init__(self):

        self.func_list = []
        self.scheduler = BackgroundScheduler()

    def timing_server(self, trigger, **kwargs):
        """
            使用示例：
            from queue_server.timing_server import TimingServer

            a = TimingServer()

            @a.timing_server(trigger="date", run_date="2019-11-06 23:13:40")
            def work():
                print("ssss")

            a.start()

            trigger q触发器取值分三种类型：
            1、 date
                在特定的时间点触发1次操作，示例如下：
                def job_func(text):
                    print(text)
                scheduler = BackgroundScheduler()
                # 在 2017-12-13 时刻运行一次 job_func 方法
                scheduler.add_job(job_func, 'date', run_date=date(2017, 12, 13), args=['text'])
                # 在 2017-12-13 14:00:00 时刻运行一次 job_func 方法
                scheduler.add_job(job_func, 'date', run_date=datetime(2017, 12, 13, 14, 0, 0), args=['text'])
                # 在 2017-12-13 14:00:01 时刻运行一次 job_func 方法
                scheduler .add_job(job_func, 'date', run_date='2017-12-13 14:00:01', args=['text'])
            2、 interval
                固定时间间隔触发， 参数如下：
                weeks (int) 	间隔几周
                days (int)  间隔几天
                hours (int)	 间隔几小时
                minutes (int)  间隔几分钟
                seconds (int)  间隔多少秒
                start_date (datetime 或 str)  开始日期
                end_date (datetime 或 str)  结束日期
                timezone (datetime.tzinfo 或str)  时区
                传参示例如下：
                # 每隔两分钟执行一次 job_func 方法
                scheduler .add_job(job_func, 'interval', minutes=2)
                # 在 2017-12-13 14:00:01 ~ 2017-12-13 14:00:10 之间, 每隔两分钟执行一次 job_func 方法
                scheduler .add_job(job_func, 'interval', minutes=2, start_date='2017-12-13 14:00:01' , end_date='2017-12-13 14:00:10')
            3、 cron
                参数如下：
                year (int 或 str)  年，4位数字
                month (int 或 str)  月 (范围1-12)
                day (int 或 str)  日 (范围1-31
                week (int 或 str)  周 (范围1-53)
                day_of_week (int 或 str)  周内第几天或者星期几 (范围0-6 或者 mon,tue,wed,thu,fri,sat,sun)
                hour (int 或 str)  时 (范围0-23)
                minute (int 或 str)	 分 (范围0-59)
                second (int 或 str) 	秒 (范围0-59)
                start_date (datetime 或 str)  最早开始日期(包含)
                end_date (datetime 或 str)  最晚结束时间(包含)
                timezone (datetime.tzinfo 或str)  指定时区
                除此还支持算数表达式， 如下：
                expression field    description
                *          any      fire on every value
                */a        any      fire every a values, starting from minimum
                a-b        any      fire on any value within the a-b range(a must be smaller than b)
                a-b/c      any      fire every c value within the a-b range
                xth y      day      fire on the x-th occurrence of weekday y within the month
                last x     day      fire on the last occurrence of weedkay x within the month
                last       day      fire on the last day within the month
                x,y,z      any      fire on any matching expresion; can combine any number of any of above expression
                示例如下：
                # 在每年 1-3、7-9 月份中的每个星期一、二中的 00:00, 01:00, 02:00 和 03:00 执行 job_func 任务
                scheduler .add_job(job_func, 'cron', month='1-3,7-9',day='0, tue', hour='0-3')

        :param str trigger: date, interval, cron
        :param dict kwargs:
        :return:
        """
        def timing(_func):
            job = Job(_func, trigger, times=kwargs)
            self.func_list.append(job)
            return _func

        return timing

    def start(self):
        for i in self.func_list:
            self.scheduler.add_job(i.func, trigger=i.trigger, **i.times)
        self.scheduler.start()


class Job(object):
    def __init__(self, _func, trigger=None, times=None):
        self.func = _func
        self.trigger = trigger
        self.times = times or {}



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/22 9:57
# @Author  : wiken
# @Site    : 
# @File    : system_service.py
# @Software: PyCharm
# @Desc    :

from datetime import datetime
import requests


BASE_URL = "http://192.168.0.100:5000"


def now():
    url = BASE_URL + "/api/System/GetServerInfo"
    res = requests.get(url)
    return parse_time(res.json())


def parse_time(time_dic):
    if time_dic.get("isSuccess"):
        _time = time_dic.get("data", {}).get("serverTime")
        _time = _time[:19]
        a = datetime.strptime(_time, "%Y-%m-%dT%H:%M:%S")
        return a
    else:
        return None

if __name__ == '__main__':
    print(now())


#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/7 9:53
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : __init__.py.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
from ys_service.queue_service import BlockService
from ys_service.queue_service.context import Context
from ys_service.host import Host


ys_host = Host("ys", "ysmq", "192.168.0.100", thread_num=10, log_level=None)



__all__ = ["ys_host", "Context"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/4 14:00
# @Author  : 7913
# @Site    : https://github.com/v5yangzai
# @File    : server.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
import logging
from pika.channel import Channel
from typing import Callable, Optional
from concurrent.futures import ThreadPoolExecutor

from ys_service.queue_service.context import Context
from ys_service.queue_service.queue import Queue
from ys_service.queue_service.action import *
from ys_service.queue_service.base_service import BaseService


class BlockService(BaseService):

    def __init__(
            self,
            username: str,
            password: str,
            host: str,
            port: int = 15672,
            heartbeat: int = 5,
            prefetch: int = 3,
            thread_num: Optional[int] = None,
            log_level: Optional[int] = logging.INFO,
            log_format: Optional[str] = None,
    ):
        super().__init__(username, password, host, port, heartbeat, prefetch, log_level, log_format)
        self.thread_num = thread_num
        self._thread_executor = ThreadPoolExecutor(max_workers=thread_num)

    def connect_queue(self):
        """
        连接到队列上
        :return:
        """
        self.logger.info("正在与队列建立连接")
        credentials = pika.PlainCredentials(self.username, self.password)
        parameters = pika.ConnectionParameters(
            self.host,
            credentials=credentials,
            heartbeat=self.heartbeat
        )

        connection = pika.BlockingConnection(parameters=parameters)
        self.logger.info("队列连接成功")
        self.connection = connection

    def listen_queue(self):
        """
        监听到注册的队列
        :return:
        """
        # 创建channel
        channel = self.connection.channel()
        for _, queue in self.queue_map.items():
            print("queue name",queue.queue_name)
            channel.basic_qos(prefetch_count=queue.prefetch)
            channel.basic_consume(
                queue=queue.queue_name,
                on_message_callback=self.on_message(queue.func, queue),
                auto_ack=queue.auto_ack,
            )
            self.logger.info(f"队列<{queue.queue_name}>监听中...")
        self.logger.info("开始消费")
        try:
            channel.start_consuming()
        except KeyboardInterrupt:
            self.logger.error("中断消费。。。")
            channel.stop_consuming()

    def on_message(self, func: Callable, queue: Queue):
        """
        用于并发处理消息
        :param func: 处理函数
        :param queue: 队列对象
        :return:
        """
        func = partial(self.handler_func, func, queue)
        return func

    def handler_func(
            self,
            func: Callable,
            queue: Queue,
            channel: Channel,
            method_frame,
            header_frame,
            body: bytes
    ):
        context = self.get_context(queue, channel, method_frame, header_frame, body, queue.service)
        self._thread_executor.submit(self.handler, func, context)

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):
        self.logger.error("队列关闭")
        self.connection.close()

    def start(self):
        self.connect_queue()
        self.listen_queue()

    @staticmethod
    def handler(func: Callable, context: Context):
        """
        jie
        :param func: 处理函数
        :param context: Context对象
        :return:
        """
        result = func(context)
        result()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/20 9:48
# @Author  : wiken
# @Site    : 
# @File    : web_service_client.py
# @Software: PyCharm
# @Desc    :
from suds.client import Client


class CreateWebServiceClient(object):
    def create(self, url):
        client = Client(url)
        return client



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/11 16:54
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : mongo.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
from bson.json_util import dumps, loads, ObjectId
from bson.decimal128 import Decimal128
from decimal import InvalidOperation
from pymongo import MongoClient
from pymongo.bulk import BulkOperationBuilder
from ys_service.db_service.base_service import BaseDB


DEFAULT_MONGO_HOST = ""
DEFAULT_MONGO_PORT = ""
DEFAULT_MONGO_USERNAME = ""
DEFAULT_MONGO_PASSWORD = ""
DEFAULT_MONGO_DB = ""


class MongoService(BaseDB):

    def connect(self, host=None, port=None, username=None, password=None, db=None):
        """
        获取mongo连接对象
        :param host:
        :param port:
        :param username:
        :param password:
        :param db:
        :return:
        """
        _host = host if host else DEFAULT_MONGO_HOST or None
        _port = port if port else DEFAULT_MONGO_PORT or None
        _username = username if username else DEFAULT_MONGO_USERNAME or None
        _password = password if password else DEFAULT_MONGO_PASSWORD or None
        _db = db if db else DEFAULT_MONGO_DB or None
        hash_value = self.get_hash(_host, _port, _username, _password, _db)
        # 如果不存在先创建连接
        if not self.is_in_map(hash_value):
            conn = MongoClient(host=_host, port=port, username=_username, password=_password)
            self.add_to_map(hash_value, conn)

        return self.get_db(hash_value, _db)

    def get_db(self, hash_value, db):
        """
        获取db对象
        :param hash_value:
        :param db:
        :return:
        """
        return self.connect_map[hash_value][db]

    def connect_ys(self, user, password, db):
        uri = f"mongodb://{user}:{password}@192.168.0.100:27017,192.168.0.100:27018/?replicaSet=rs0"

        hash_value = self.get_hash(uri)
        if not self.is_in_map(hash_value):
            conn = MongoClient(host=uri, replicaSet="rs0")
            self.add_to_map(hash_value, conn)
        return self.get_db(hash_value, db=db)

    @staticmethod
    def bulk(collection, ordered=True, passive_document_validation=False):
        """
        批量执行
        :param collection: 操作
        :param ordered: True所有命令将按顺序执行，并且第一个出错就会中止整个流程，False将按
                        任意顺序执行，可能是并行的，并且报告所有的操作完成后的错误。默认为True
        :param passive_document_validation:（可选）如果为True，则允许写入选择退出文档级别的验证。
                        默认值为 False。
        :return:
        """
        return BulkOperationBuilder(collection, ordered, passive_document_validation)

    @staticmethod
    def dumps(value):
        return dumps(value)

    @staticmethod
    def loads(value):
        return loads(value)

    @staticmethod
    def object_id(s):
        return ObjectId(s)

    @staticmethod
    def decimal128(s):
        """
        浮点数或者字符串转化为decimal128
        :param s:
        :return:
        """
        if isinstance(s, Decimal128):
            return s

        try:
            money = Decimal128(s)
        except (ValueError, InvalidOperation):
            money = Decimal128("0")
        return money



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/4 13:59
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : __init__.py.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
import logging
from typing import Optional
from ys_service.queue_service.block_service import BlockService


def get_block_server(
            username: str,
            password: str,
            host: str,
            port: int = 15672,
            heartbeat: int = 5,
            prefetch: int = 3,
            thread_num: Optional[int] = None,
            log_level: Optional[str] = logging.INFO,
            log_format: Optional[str] = None,
):

    return BlockService(
        username, password, host, port,
        heartbeat, prefetch, thread_num, log_level, log_format
    )


DEFAULT_SERVER = get_block_server("ys", "ysmq", "192.168.0.100", thread_num=10, log_level=None)



import os
import time
import codecs
import logging

from logging import FileHandler


class SafeFileHandler(FileHandler):
    def __init__(self, filename, mode='a', encoding=None, delay=0):
        """
        Use the specified filename for streamed logging
        """
        if codecs is None:
            encoding = None
        FileHandler.__init__(self, filename, mode, encoding, delay)
        self.mode = mode
        self.encoding = encoding
        self.suffix = "%Y-%m-%d"
        self.suffix_time = ""

    def emit(self, record):
        """
        Emit a record.

        Always check time
        """
        try:
            if self.check_baseFilename(record):
                self.build_baseFilename()
            FileHandler.emit(self, record)
        except (KeyboardInterrupt, SystemExit):
            raise
        except:
            self.handleError(record)

    def check_baseFilename(self, record):
        """
        Determine if builder should occur.

        record is not used, as we are just comparing times,
        but it is needed so the method signatures are the same
        """
        timeTuple = time.localtime()

        if self.suffix_time != time.strftime(self.suffix, timeTuple) or not os.path.exists(
                self.baseFilename + '.' + self.suffix_time):
            return 1
        else:
            return 0

    def build_baseFilename(self):
        """
        do builder; in this case,
        old time stamp is removed from filename and
        a new time stamp is append to the filename
        """
        if self.stream:
            self.stream.close()
            self.stream = None

        # remove old suffix
        if self.suffix_time != "":
            index = self.baseFilename.find("." + self.suffix_time)
            if index == -1:
                index = self.baseFilename.rfind(".")
            self.baseFilename = self.baseFilename[:index]

        # add new suffix
        currentTimeTuple = time.localtime()
        self.suffix_time = time.strftime(self.suffix, currentTimeTuple)
        self.baseFilename = self.baseFilename + "." + self.suffix_time

        self.mode = 'a'
        if not self.delay:
            self.stream = self._open()


class Logger(object):
    level_relations = {
        'debug': logging.DEBUG,
        'info': logging.INFO,
        'warning': logging.WARNING,
        'error': logging.ERROR,
        'crit': logging.CRITICAL
    }

    def __init__(self, filename, level,
                 fmt='%(asctime)s - %(pathname)s[line:%(lineno)d] - %(levelname)s: \n%(message)s'):

        path = "{}".format(os.sep).join(os.getcwd().split("{}".format(os.sep)))
        self.log_file_exist(path)
        self.filename = os.path.join(path, filename)
        if level == 'info':
            self.logger = logging.getLogger(self.filename)
            format_str = logging.Formatter(fmt)  # 设置日志格式
            self.logger.setLevel(self.level_relations.get(level))  # 设置日志级别
            # sh = logging.StreamHandler()  # 往屏幕上输出
            # sh.setFormatter(format_str)  # 设置屏幕上显示的格式
            th = SafeFileHandler(filename=self.filename, encoding='utf-8')
            # th.suffix = "%Y-%m-%d.log"
            th.setFormatter(format_str)  # 设置文件里写入的格式
            # self.logger.addHandler(sh)  # 把对象加到logger里
            self.logger.addHandler(th)
        else:
            self.logger = logging.getLogger(self.filename)
            format_str = logging.Formatter(fmt)  # 设置日志格式
            self.logger.setLevel(self.level_relations.get(level))  # 设置日志级别
            # sh = logging.StreamHandler()  # 往屏幕上输出
            # sh.setFormatter(format_str)  # 设置屏幕上显示的格式
            th = SafeFileHandler(filename=self.filename, encoding='utf-8')
            # th.suffix = "%Y-%m-%d.log"
            th.setFormatter(format_str)  # 设置文件里写入的格式
            self.logger.addHandler(th)  # 把对象加到logger里

    def log_file_exist(self, path):
        _ = self
        if not os.path.exists(os.path.join(path, 'logs')):
            os.mkdir(os.path.join(path, 'logs'))


info_log = Logger('logs/info.log', level='info')
error_log = Logger('logs/error.log', level='error')


if __name__ == '__main__':
    for i in range(10):
        # time.sleep(0.1)
        # time.sleep(1)
        print(i)
        info_log.logger.info("test: "+str(i))
        error_log.logger.error(i)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/4 14:35
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : context.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
from bson.objectid import ObjectId
from ys_service.queue_service.action import *

from ys_service.http_service.log_service import LogCreate
from ys_service.http_service.context_attach_data_server import ContextAttachDataServer
from ys_service.http_service.attach_data_service import CreateAttachDataService
from ys_service.log_service.local_log import info_log, error_log


class Context(object):
    """
    将获取的详细进行封装为Context对象
    """
    def __init__(self, queue, logger, connection, channel, method_frame, header_frame, body, service):
        self.queue = queue
        self.logger = logger
        self.conn = connection
        self.chan = channel
        self.method_frame = method_frame
        self.header_frame = header_frame
        self.body = body
        self.timing = None  # 定时
        self.service = service
        self.object_id = ObjectId
        self.logger = None  # 日志对像
        self._context_attach_data_handler = None

    @property
    def delivery_tag(self):
        return self.method_frame.delivery_tag

    def ack(self, reply_data=None):
        return ActionAck(self, reply_data)()

    def nack(self):

        return ActionNack(self)()

    def next_to(
            self,
            exchange_name=None,
            routing_key=None,
            new_message_body=None,
            new_message_headers=None,
    ):
        return ActionNextTo(self, exchange_name, routing_key, new_message_body, new_message_headers)()

    def go_back(self, timing=None):
        return ActionGoBack(self, timing)()

    def get_header(self, key):
        return self.header_frame.headers.get(key)

    @property
    def text(self):
        """
        获取数据的字符串格式
        :return:
        """
        return self.body.decode("utf-8") if self.body else ""

    def get_json(self):
        """
        获取数据反序列化后的对象
        :return: 反序列化后的对象
        """
        try:
            return json.loads(self.text)
        except json.JSONDecodeError:
            return None

    @property
    def attach_data(self):

        if not self._context_attach_data_handler:
            attach_data_handler = CreateAttachDataService().create()
            try:
                message_id = self.get_json()["messageId"]
            except (TypeError, KeyError) as e:
                return {
                    "is_success": False,
                    "status_code": "",
                    "message": "从队列中获取message_id失败, 异常信息{}".format(e),
                    "data": self.body
                }
            self._context_attach_data_handler = ContextAttachDataServer(
                message_id=message_id,
                handler=attach_data_handler
            )

        return self._context_attach_data_handler

    def log(self, message, level="info", data_property=None, version=None, auth=None,
            application_name=None, application_module=None):

        if not self.logger:
            application = {
                "applicationName": "py_host: " + (application_name or self.queue.queue_name),
                "applicationVersion": version,
                "applicationModule": application_module,
                "author": auth
            }

            trace_property = {
                "traceId": self.header_frame.headers.get("trace_id"),
                "processId": self.header_frame.headers.get("process_id"),
                "processStage": self.header_frame.headers.get("process_stage")
            }
            if not self.logger:
                self.logger = LogCreate().create(application, trace_property)

        return self.logger.send_log(message,
                                    level=level,
                                    data_property=data_property)

    def local_log(self, msg, level="info"):
        """
        将日志打印到本地
        :param msg: 日志内容
        :param level: 日志等级 支持info、 error
        :return: None
        """
        log_info = "传入数据:{}\n返回数据:{}\n".format(self.get_json(), msg)
        if level == "info":
            info_log.logger.info(log_info)
        else:
            error_log.logger.error(log_info)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/29 11:24
# @Author  : wiken
# @Site    : 
# @File    : __init__.py.py
# @Software: PyCharm
# @Desc    :


if __name__ == "__main__":
    ...



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/8 10:47
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : ys_hos.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
import logging
from functools import partial
from typing import Optional
from apscheduler.schedulers.blocking import BlockingScheduler
from ys_service.http_service.system_service import now
from ys_service.queue_service.block_service import BlockService
from ys_service.timing_service.timing_service import TimingService
from ys_service.service_provider import ServiceProvider

FLAG = """

$$$$$$$\ $$\     $$\ $$$$$$$$\ $$\   $$\  $$$$$$\  $$\   $$\       $$\   $$\  $$$$$$\   $$$$$$\ $$$$$$$$\ 
$$  __$$\\$$\   $$  |\__$$  __|$$ |  $$ |$$  __$$\ $$$\  $$ |      $$ |  $$ |$$  __$$\ $$  __$$\\__$$  __|
$$ |  $$ |\$$\ $$  /    $$ |   $$ |  $$ |$$ /  $$ |$$$$\ $$ |      $$ |  $$ |$$ /  $$ |$$ /  \__|  $$ |   
$$$$$$$  | \$$$$  /     $$ |   $$$$$$$$ |$$ |  $$ |$$ $$\$$ |      $$$$$$$$ |$$ |  $$ |\$$$$$$\    $$ |   
$$  ____/   \$$  /      $$ |   $$  __$$ |$$ |  $$ |$$ \$$$$ |      $$  __$$ |$$ |  $$ | \____$$\   $$ |   
$$ |         $$ |       $$ |   $$ |  $$ |$$ |  $$ |$$ |\$$$ |      $$ |  $$ |$$ |  $$ |$$\   $$ |  $$ |   
$$ |         $$ |       $$ |   $$ |  $$ | $$$$$$  |$$ | \$$ |      $$ |  $$ | $$$$$$  |\$$$$$$  |  $$ |   
\__|         \__|       \__|   \__|  \__| \______/ \__|  \__|      \__|  \__| \______/  \______/   \__|   

"""


class Host(object):
    """
    云上host
    """
    def __init__(
            self,
            username: str,
            password: str,
            host: str,
            port: int = 15672,
            heartbeat: int = 5,
            prefetch: int = 3,
            thread_num: Optional[int] = None,
            log_level: Optional[int] = logging.INFO,
            log_format: Optional[str] = None,
    ):
        # 队列服务

        self.queue_server = BlockService(
            username, password, host,
            port, heartbeat, prefetch, thread_num,
            log_level, log_format
        )
        # 定时服务
        self.time_server = TimingService()
        # 数据库等service提供者
        self.service = ServiceProvider()

    def register(self, queue_name, auto_ack=False, prefetch=3, config_key=None):
        """
        注册队列名字
        :param queue_name: 队列名
        :param auto_ack: 是否自动回复
        :param prefetch: 流量数
        :param config_key: 配置中心的key
        :return:
        """
        def _inner(f):
            return self.queue_server.register(queue_name, auto_ack, prefetch, self.service)(f)
        return _inner

    def timing(self, trigger, **kwargs):
        """
        注册定时时间
        :param trigger:
        :param kwargs:
        :return:
        """
        def _inner(f):
            _f = partial(f, service=self.service)
            return self.time_server.timing_server(trigger, **kwargs)(_f)
        return _inner

    @property
    def queue_map(self):
        """
        获取queue_map
        :return:
        """
        return self.queue_server.queue_map

    def start(self):

        print(FLAG)
        self.start_only_timing()
        self.start_only_queue()

    def start_only_queue(self):
        BlockService.start(self.queue_server)

    def start_only_timing(self):
        if not self.queue_map:
            self.time_server.scheduler = BlockingScheduler()
        TimingService.start(self.time_server)
        # BlockService.start(self.queue_server)

    @staticmethod
    def now():
        return now()

    def __enter__(self):
        return self

    def __exit__(self, exc_type, exc_val, exc_tb):

        for _, v in self.service.mongo_client.connect_map.items():
            v.close()

        for _, v in self.service.mysql_client.connect_map.items():
            v.close()

        for _, v in self.service.redis_client.connect_map.items():
            v.close()

        for _, v in self.service.sql_server_client.connect_map.items():
            v.close()

        self.queue_server.connection.close()
        print("host网络连接关闭完毕")



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/4 14:45
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : queue.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :

from typing import Callable


class Queue(object):
    """
    监听的队列的实例
    """
    def __init__(
            self,
            queue_name: str,
            func: Callable,
            auto_ack: bool,
            prefetch: int,
            service
    ):
        """
        :param queue_name: 队列名
        :param func: 处理函数
        :param auto_ack: 是否自动回复队列
        :param prefetch: 队列流量数
        :param service: 服务
        """
        self.queue_name = queue_name
        self.func = func
        self.auto_ack = auto_ack
        self.prefetch = prefetch
        self.service = service



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/12 15:20
# @Author  : wiken
# @Site    : 
# @File    : log_service.py
# @Software: PyCharm
# @Desc    :
import time
from ys_service.http_service._http import _HTTP


class LogService(_HTTP):
    def __init__(self, application_property, trace_property):
        super(LogService, self).__init__()

        self.application_property = application_property
        self.trace_property = trace_property

    def send_log(self, message, data_property=None, level="info"):
        """
        send log to log_center
        :param data_property:
        :param message:
        :param level: The level of message.
        :return: response
        """

        data = {
            "timestamp": time.strftime("%Y-%m-%d %H:%M:%S"),
            "level": level,
            "message": message,
            "properties": {
                "applicationProperty": self.application_property,
                "dataProperty": data_property,
                "traceProperty": self.trace_property
            }
        }

        url = self.base_url + "/api/LogCenter/NewLog"

        res = self.request.post(url, json=data)
        return res


class LogCreate(object):

    @staticmethod
    def create(application_properties=None, trace_property=None):
        """
        传入基础数据, 数据详情请见：http://192.168.0.100:5000/swagger/index.html
        :param application_properties:
        :param trace_property:
        :return: log_service 对象
        """

        return LogService(application_properties, trace_property)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/14 16:32
# @Author  : wiken
# @Site    : 
# @File    : realtime_notice.py
# @Software: PyCharm
# @Desc    :
from ys_service.http_service.utils import *
from ys_service.http_service._http import _HTTP


class CreateRealtimeNoticeService(object):
    """
        创建实时消息服务
    """
    @property
    def create(self):
        return RealtimeNoticeService


class RealtimeNoticeService(_HTTP):
    """
        实时消息
    """
    def new_notice(self, data):
        url = self.base_url + "/api/RealtimeNotice/NewNotice"
        response = self.request.post(url, json=data)

        return check_http_response(response)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/5 15:57
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : action.py
# @project : ys_module
# @Software: PyCharm
# @Desc    : 回复的动作
import uuid
import pika
import json
from functools import partial
from typing import Union


class Action(object):

    def __init__(self, context):
        self.context = context
        self.queue = context.queue
        self.body = context.body
        self.method_frame = context.method_frame
        self.header_frame = context.header_frame
        self.chan = context.chan
        self.conn = context.conn

    def _publish(
            self,
            exchange_name=None,
            routing_key=None,
            new_message: Union[str, bytes]=None,
            properties=None,
    ):
        """
        生产消息
        :param exchange_name:
        :param routing_key:
        :param new_message: 新消息内容
        :param properties:
        :return:
        """
        body = self.get_body(new_message)
        self.conn.add_callback_threadsafe(
            partial(self.chan.basic_publish, exchange_name, routing_key, body, properties=properties)
        )

    @staticmethod
    def get_body(data):
        """
        返回数据的二进制格式
        :param data:
        :return:
        """

        if isinstance(data, bytes):
            return data
        try:
            return json.dumps(data, ensure_ascii=False).encode("utf-8")
        except json.JSONDecodeError:
            return data.encode("utf-8")

    @property
    def delivery_tag(self):
        return self.context.delivery_tag


class ActionAck(Action):

    exchange = "system.response"

    def __init__(self, context, reply_data=""):
        super().__init__(context)
        self.reply_data = reply_data

    def __call__(self, *args, **kwargs):
        reply_to = self.header_frame.reply_to

        # 当有replay_to的时候，向replay_to发送信息
        if reply_to:
            properties = pika.BasicProperties(
                correlation_id=self.header_frame.correlation_id,
                message_id=str(uuid.uuid1())
            )
            self._publish(self.exchange, reply_to, self.reply_data, properties=properties)

        self.conn.add_callback_threadsafe(partial(self.chan.basic_ack, self.delivery_tag))


class ActionNack(Action):

    def __call__(self, *args, **kwargs):
        self.conn.add_callback_threadsafe(partial(self.chan.basic_nack, self.delivery_tag))


class ActionNextTo(Action):

    def __init__(self, context, exchange_name, routing_key, new_message_data, new_message_headers):
        Action.__init__(self, context)
        self.exchange_name = exchange_name
        self.routing_key = routing_key
        self.new_message_data = new_message_data
        self.new_message_headers = new_message_headers

    def __call__(self, *args, **kwargs):
        # 当没有新的消息header时，copy原来的headers
        headers = self.get_headers()
        data = self.get_body(self.new_message_data) if self.new_message_data else self.context.body
        properties = pika.BasicProperties(
            headers=headers,
        )
        self._publish(self.exchange_name, self.routing_key, data, properties)
        self.conn.add_callback_threadsafe(partial(self.chan.basic_ack, self.delivery_tag))

    def get_headers(self):
        """
        获取头部信息, 并且设定processID
        :return:
        """
        old_header = getattr(self.context.header_frame, "headers", {})
        if not self.new_message_headers:
            headers = old_header
        else:
            headers = self.new_message_headers
            for i, v in old_header.items():
                # 当新头部中有与context的header中相同的字段的时候，不作改变
                if headers.get(i):
                    continue
                # 当新头中没有i键的时候，设置新头的i键的值为v
                headers[i] = v
        headers["processID"] = str(uuid.uuid1())
        headers["traceID"] = old_header.get("traceID")
        return headers


class ActionGoBack(Action):

    def __init__(self, context, timing=None):
        """
        :param context:
        :param timing: 延迟执行或者定时执行时间表达式
        """
        super().__init__(context)
        self.timing = timing

    def __call__(self, *args, **kwargs):
        self._publish()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/14 16:06
# @Author  : wiken
# @Site    : 
# @File    : reservation_service.py
# @Software: PyCharm
# @Desc    :
from ys_service.http_service.utils import *
from ys_service.http_service._http import _HTTP


class CreateReservation(object):
    """
    创建预约服务类
    """

    @property
    def create(self):
        return ReservationService


class ReservationService(_HTTP):
    """
    本类实现队列的预约操作
    """
    def check_in(self, data):
        url = self.base_url + "/api/Reservation/CheckIn"
        res = self.request.post(url, json=data)

        return check_http_response(res)

    def query(self, data):
        url = self.base_url + "/api/Reservation/Query"
        res = self.request.post(url, json=data)

        return check_http_response(res)

    def cancel(self, data):
        url = self.base_url + "/api/Reservation/Cancel"
        res = self.request.post(url, json=data)

        return check_http_response(res)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/12 14:45
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : http_service.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :

import requests


class HTTPService(object):

    def __init__(self):
        pass

    # @property
    # def log_create(self):
    #     """
    #     创建 log_center 对象
    #     :return: 实例对象
    #     """
    #
    #     return LogCreate().create
    #
    # @property
    # def mq_create(self):
    #     """
    #     创建 mq 对象
    #     :return: 实例对象
    #     """
    #
    #     return CreateMQ().create




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/14 16:10
# @Author  : wiken
# @Site    : 
# @File    : utils.py
# @Software: PyCharm
# @Desc    :
import hashlib
from json.decoder import JSONDecodeError


def check_http_response(res):
    """
    检查返回的http response 是否是需要的数据
    :param res:
    :return: dict
    """
    if isinstance(res, dict):
        return res
    try:
        dic = res.json()
    except (AttributeError, JSONDecodeError) as e:
        print(e)
        return None

    else:
        return dic


def return_msg(is_success, status_code=None, msg=None, data=None):
    return {
        "is_success": is_success,
        "status_code": status_code,
        "message": msg,
        "data": data
    }


def convert_time(_date):
    # '11/19/19 11:48:50 AM'
    import datetime
    if _date.count("/") == 2 and _date.count(":") == 2:
        date = _date[:-3]
        date = datetime.datetime.strptime(date, "%m/%d/%y %H:%M:%S")
        if "PM" in _date:
            date += datetime.timedelta(hours=12)
        return date.strftime("%Y-%m-%d %H:%M:%S")
    else:
        return _date


def convert_to_lower(dic):
    """
    把字典中地健值全部转换为小写字母
    :param dic:
    :return:
    """
    lower_dic = {}
    for k, v in dic.items():
        if isinstance(v, list):
            temp_v = []
            for i in v:
                temp_d = {}
                for k2, v2 in i.items():
                    # print(v2)
                    temp_d[k2.lower()] = v2
                temp_v.append(temp_d)
            f_v = temp_v
        elif isinstance(v, dict):
            temp_v = {}
            for k2, v2 in v.items():
                temp_v[k2.lower()] = v2
            f_v = temp_v
        else:
            f_v = v
        lower_dic[k.lower()] = f_v
    return lower_dic


def md5_encrypt(info):
    """
    md5 加密
    :param info:
    :return:
    """
    # s = "15922908607"
    a = hashlib.md5()
    a.update(info.encode(encoding='utf-8'))
    return a.hexdigest()
# print(convert_time('11/19/19 11:48:50 PM'), type(convert_time('11/19/19 11:48:50 PM')))


__all__ = ["md5_encrypt", "convert_to_lower", "convert_time", "check_http_response", "return_msg"]



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/14 16:33
# @Author  : wiken
# @Site    : 
# @File    : config_center.py
# @Software: PyCharm
# @Desc    :
from ys_service.http_service.utils import *
from ys_service.http_service._http import _HTTP


class CreateConfigCenterService(object):
    """
        创建配置中心服务
    """
    @property
    def create(self):
        return ConfigCenterService


class ConfigCenterService(_HTTP):
    """
    创建配置中心实例
    """
    def push(self, data):
        url = self.base_url + "/api/ConfigCenter/Push"
        response = self.request.post(url, data)

        return check_http_response(response)

    def active(self, data):
        url = self.base_url + "/api/ConfigCenter/Active"
        response = self.request.post(url, data)

        return check_http_response(response)

    def pull(self, data):
        url = self.base_url + "/api/ConfigCenter/Pull"
        response = self.request.post(url, data)

        return check_http_response(response)

    def fetch(self, data):
        url = self.base_url + "/api/ConfigCenter/Fetch"
        response = self.request.post(url, data)

        return check_http_response(response)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/14 9:51
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : sqlserver_service.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :

import pymssql
from ys_service.db_service.base_service import BaseDB

DEFAULT_SQL_SERVER_HOST = ""
DEFAULT_SQL_SERVER_USERNAME = ""
DEFAULT_SQL_SERVER_PASSWORD = ""
DEFAULT_SQL_SERVER_DB = ""


class SqlServerService(BaseDB):

    def connect(self, host=None, username=None, password=None, db=None):
        """
        创建连接对象，并返回对应游标
        :param host: mysql服务器地址
        :param username: 用户名
        :param password: 密码
        :param db: 数据库
        :return: 游标
        """
        _host = host if host else DEFAULT_SQL_SERVER_HOST or None
        _username = username if username else DEFAULT_SQL_SERVER_USERNAME or None
        _password = password if password else DEFAULT_SQL_SERVER_PASSWORD or None
        _db = db if db else DEFAULT_SQL_SERVER_DB or None
        # 获取数据库连接参数的hash值, 作为连接对象的键
        hash_value = self.get_hash(_host, _username, _password, _db)
        if not self.is_in_map(hash_value):
            conn = pymssql.connect(
                server=_host, user=_username, password=_password, database=_db
            )
            self.add_to_map(hash_value, conn)

        return self.get_cursor(hash_value)

    def get_cursor(self, hash_value):
        """
        获取游标对象
        :param hash_value:
        :return:
        """
        return self.connect_map[hash_value].cursor()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/15 11:57
# @Author  : wiken
# @Site    : 
# @File    : attach_data_service.py
# @Software: PyCharm
# @Desc    : 实现通过data_id 和data_type 把数据通过键值对的形式或附加的存储到数据库中，实现新加、追加、查询、等操作。返回数据类统一为json格式
import json
import datetime

from ys_service.http_service.utils import *
from ys_service.http_service._http import _HTTP


class AttachDataService(_HTTP):
    def get(self, data_type, data_id, timeout=30, props=None):
        """
        通过props获取attach 中的数据
        :param data_type: 数据类型（如:YATP）
        :param data_id: 数据唯一id
        :param timeout: 超时时间
        :param props: 需要获取值的健的列表，
        :return: 得到的数据， dict类型
        """

        url = self.base_url + "/api/MQ/SendRequest"

        data = {
            "timeoutSeconds": timeout,
            "exchangeName": "system.request",
            "routingKey": "system.request.attach_data",
            "requestHeaders": {
                "message_type": "get_prop" if props else "get_props",
                "data_type": data_type,
                "data_id": data_id
            },
            "requestData": ""}

        response = self.request.post(url=url, json=data)

        return check_http_response(response)

    def set(self, props, data_type, data_id, timeout=30):
        """
        根据id设置属性值
        :param timeout:
        :param props: 需要设置的属性值
        :param str data_type: 数据类型（如:YATP）
        :param str data_id: 数据唯一id
        :return: 设置是否成功返回值
        """

        url = self.base_url + "/api/MQ/SendRequest"

        data = {
          "timeoutSeconds": timeout,
          "exchangeName": "system.request",
          "routingKey": "system.request.attach_data",
          "requestHeaders": {
            "message_type": "set_prop",
            "data_type": data_type,
            "data_id": data_id
          },
          "requestData": json.dumps(props),
          "executionTime": datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f")[0:23] + "Z"
        }
        response = self.request.post(url, json=data)

        return check_http_response(response)

    def append(self, data, data_type, data_id):
        """
        在属性后附加数据, 附加的数据以字典的格式传入，在数据库中存在attachList 中，
        可以使用 get_all 方法获取。
        :param data: 需要附加的内容， 可以是任何形式的值。
        :param str data_type: 数据类型（如:YATP）
        :param str data_id: 数据唯一id
        :return: 设置是否成功返回值
        """

        url = self.base_url + "/api/MQ/SendRequest"

        data = {
            "exchangeName": "system.request", "routingKey": "system.request.attach_data",
            "requestHeaders": {
                "message_type": "append_data",
                "data_type": data_type,
                "data_id": data_id
            },
            "requestData": json.dumps(data),
            # "requestData": data,
            "executionTime": datetime.datetime.now().strftime("%Y-%m-%dT%H:%M:%S.%f")[0:23] + "Z"}

        response = self.request.post(url, json=data)
        return check_http_response(response)

    def get_all(self, data_type, data_id, timeout=30):
        """
        根据data_type 和 data_id 获取所有的prop值和 attachList 中的值
        :param str data_type: 数据类型（如:YATP）
        :param str data_id: 数据唯一id
        :param timeout: 超时时间
        :return: 设置是否成功返回值
        """

        url = self.base_url + "/api/MQ/SendRequest"
        data = {
          "timeoutSeconds": timeout,
          "exchangeName": "system.request",
          "routingKey": "system.request.attach_data",
          "requestHeaders": {
            "message_type": "query_data",
            "data_type": data_type,
            "data_id": data_id
          },
          "requestData": ""
        }
        response = self.request.post(url=url, json=data)
        dic = check_http_response(response)

        len_of_attach_lst = len(dic.get("Data", {}).get("AttachList"))
        for i in range(len_of_attach_lst):
            dic["Data"]["AttachList"][i]["AttachTime"] = convert_time(
                dic.get("Data", {}).get("AttachList")[i].get("AttachTime")
            )
        return dic


class CreateAttachDataService(object):
    @staticmethod
    def create():
        return AttachDataService()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/5 13:38
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : server.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :

import logging
from typing import Optional, Callable
from ys_service.queue_service.queue import Queue
from ys_service.queue_service.context import Context


class BaseService(object):

    DEFAULT_LOG_FORMAT = "%(levelname) -5s %(asctime)-10s %(lineno) -5d: %(message)s"

    def __init__(
            self,
            username: str,
            password: str,
            host: str,
            port: int = 15672,
            heartbeat: int = 5,
            prefetch: int = 3,
            log_level: Optional[int] = logging.INFO,
            log_format: Optional[str] = None,
    ):
        """
        :param username: 队列用户名
        :param password: 队列密码
        :param host: 队列主机地址
        :param port: 队列主机端口
        :param heartbeat: 心跳
        :param prefetch: 队列流量数
        :param log_level: 日志等级
        :param log_format: 日志格式
        """
        self.username = username
        self.password = password
        self.host = host
        self.port = port
        self.heartbeat = heartbeat
        self.prefetch = prefetch
        # 队列名与处理函数的映射体
        self.queue_map = {}
        # 队列连接对象
        self.connection = None
        # 频道对象
        self.channel = None
        self.logger = self.create_logger(log_level, log_format)

    def create_logger(self, log_level, log_format):
        """
        创建日志对象
        :param log_level: 日志等级
        :param log_format: 日志格式
        :return:
        """
        log_format = self.DEFAULT_LOG_FORMAT if not log_format else log_format
        logger = logging.getLogger(__name__)
        logging.basicConfig(level=log_level, format=log_format)
        return logger

    def add_to_queue_map(self, queue_name: str, func: Callable, auto_ack: bool, prefetch: int, service):
        """
        添加队列名与执行函数的映射到 queue_map 中去
        :param queue_name: 队列名
        :param func: 执行函数
        :param auto_ack: 是否自动回复队列
        :param prefetch: 队列流量数
        :return:
        """
        queue = Queue(queue_name, func, auto_ack=auto_ack, prefetch=prefetch, service=service)
        self.queue_map[queue_name] = queue

    def register(
            self,
            queue_name: str,
            auto_ack: bool = False,
            prefetch: int = 3,
            service=None,
    ):
        """
        注册队列名及其执行函数
        :param queue_name: 队列名
        :param auto_ack: 是否自动回复队列
        :param prefetch: 流量数
        :param service: 流量数
        :return:
        """

        def _register(func: Callable):
            """
            :param func: 执行函数
            :return:
            """
            self.add_to_queue_map(queue_name, func, auto_ack, prefetch, service)
            return func

        return _register

    def get_context(
            self,
            queue: Queue,
            channel,
            method_frame,
            header_frame,
            body: bytes,
            service

    ) -> Context:
        """
        将获取的消息进行封装，等到context对象
        :return: context实例
        """
        context = Context(queue, self.logger, self.connection, channel, method_frame, header_frame, body, service)

        return context



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/11 17:19
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : service_provider.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
import json
import time
from decimal import Decimal
from datetime import datetime, timedelta
from ys_service.db_service import DBService, MongoService, MysqlService, RedisService, SqlServerService

from ys_service.http_service.mq_service import CreateMQ
from ys_service.http_service.log_service import LogCreate
from ys_service.http_service.http_service import HTTPService
from ys_service.http_service.attach_data_service import CreateAttachDataService
from ys_service.http_service.reservation_service import CreateReservation
from ys_service.http_service.config_center_service import CreateConfigCenterService
from ys_service.http_service.realtime_notice_service import CreateRealtimeNoticeService
from ys_service.http_service.web_service_client import CreateWebServiceClient
from ys_service.http_service.ys_requests import YsRequests


class ServiceProvider(object):

    def __init__(self):
        # 数据库对象
        self.db_service = DBService()
        self.http_service = HTTPService()
    @property
    def mongo_client(self) -> MongoService:
        """
        获取mongo服务
        :return:
        """
        return self.db_service.mongo_client

    @property
    def mysql_client(self) -> MysqlService:
        """
        获取mysql服务
        :return:
        """
        return self.db_service.mysql_client

    @property
    def redis_client(self) -> RedisService:
        """
        获取redis服务
        :return:
        """
        return self.db_service.redis_client

    @property
    def sql_server_client(self) -> SqlServerService:
        return self.db_service.sql_server_client

    def strptime(self, time_str, f=None):
        """
        将时间字符串格式化为时间类型
        :param f: 时间格式
        :param time_str: 时间字符串
        :return:
        """
        if f:
            return datetime.strptime(time_str, f)

        try:
            return datetime.strptime(time_str, "%Y-%m-%d %H:%M:%S")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, "%Y/%m/%d %H:%M:%S")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, "%Y-%m-%d")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, "%Y/%m/%d")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, "%Y-%m-%d %H:%M:%S AM")
        except ValueError:
            pass

        try:
            d = datetime.strptime(time_str, "%Y/%m/%d %H:%M:%S PM")
            return d + self.timedelta(hours=12)
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, "%Y-%m-%d %H:%M:%S.%f0")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, "%Y/%m/%d %H:%M:%S.%f0")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, "%m/%d/%Y %H:%M:%S")
        except ValueError:
            pass

        try:
             return datetime.strptime(time_str, f"%m-%d-%y %H:%M:%S AM")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, f"%m/%d/%y %H:%M:%S AM")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, f"%m-%d-%y %H:%M:%S PM") + self.timedelta(hours=12)
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, f"%m/%d/%y %H:%M:%S PM") + self.timedelta(hours=12)
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, f"%Y-%m-%d %H:%M")
        except ValueError:
            pass

        try:
            return datetime.strptime(time_str, f"%Y/%m/%d %H:%M")
        except ValueError:
            pass

    @property
    def time(self):
        """
        获取
        :return:
        """
        return time

    @property
    def datetime(self):
        return datetime

    @property
    def timedelta(self):
        return timedelta

    @staticmethod
    def marshal(value):
        return json.dumps(value, ensure_ascii=False)

    @property
    def now(self):
        return datetime.now()

    def get_three_code(self, s):
        redis =self.redis_client.connect(host="192.168.0.100", port=6379, password="O0qtw1wHPddwCC5T", db=0)
        d = redis.hget("System:BaseInfo:AirlineInfo", s)

        return json.loads(d).get("ThreeCode") if d else ""

    @staticmethod
    def parse_to_float(d):

        try:
            money = float(d)
        except ValueError:
            money = 0
        return money

    @staticmethod
    def parse_to_decimal(d):

        try:
            money = Decimal(d)
        except ValueError:
            money = Decimal(0)
        return money

    def log_factory(self):
        """
        创建日志中心对象
        :return:
        """
        return LogCreate()

    @property
    def mq_service(self):
        """
        创建mq服务对象
        :return:
        """
        print("CreateWebServiceClient",CreateWebServiceClient)
        return CreateMQ()

    @property
    def reservation_service(self):
        """
        创建预约服务对象
        :return:
        """
        return CreateReservation()

    @property
    def realtime_notice_service(self):
        """
        创建定时消息对象
        :return:
        """
        return CreateRealtimeNoticeService()

    @property
    def config_center_service(self):
        """
        创建配置中心对象
        :return:
        """
        return CreateConfigCenterService()

    @property
    def attach_data_service(self):
        """
        获取附加消息对象
        :return:
        """
        return CreateAttachDataService()

    @property
    def ws_client(self):
        """
        获取webservice client对象
        :return:
        """
        return CreateWebServiceClient()

    @property
    def ys_requests(self):
        return YsRequests



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/12 15:21
# @Author  : wiken
# @Site    : 
# @File    : _http.py
# @Software: PyCharm
# @Desc    :


import requests


class _HTTP(object):

    def __init__(self):
        self.base_url = "http://192.168.0.100:5000"
        self.request = requests



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/11 16:54
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : __init__.py.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :

from ys_service.db_service.mysql_service import MysqlService
from ys_service.db_service.mongo_service import MongoService
from ys_service.db_service.redis_service import RedisService
from ys_service.db_service.sqlserver_service import SqlServerService


class DBService(object):

    def __init__(self):
        # 三种连接对象
        self.mysql_client = MysqlService()
        self.mongo_client = MongoService()
        self.redis_client = RedisService()
        self.sql_server_client = SqlServerService()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/22 15:48
# @Author  : wiken
# @Site    : 
# @File    : ys_requests.py
# @Software: PyCharm
# @Desc    :
import requests
from datetime import datetime, timedelta
from ys_service.http_service.system_service import now


class YsRequests(object):
    def __init__(self, context):
        self.session = requests.Session()
        self.start_time = now()
        self._deadline = context.get_header("deadline")
        self.set_deadline()

    def set_deadline(self):
        if self._deadline:
            self._deadline = datetime.strptime(self._deadline, "%Y-%m-%d %H:%M:%S")

    @classmethod
    def create(cls, context):
        """
        创建ys_requests 实例
        :param context:
        :return: 实例对象
        """
        return cls(context)

    def get(self, *args, **kwargs):
        return self.requests("get", *args, **kwargs)

    def post(self, *args, **kwargs):
        return self.requests("post", *args, **kwargs)

    def requests(self, *args, **kwargs):
        timeout = kwargs.setdefault("timeout", 30)
        self.is_timeout(timeout)
        return self.session.request(*args, **kwargs)

    def is_timeout(self, timeout):
        """
        从队列消息中得到到期时间，如果下一次请求超时后还不到期则执返回True
        :param timeout:
        :return: bool
        """
        if self._deadline:
            if self._deadline - now() < timedelta(seconds=timeout):
                raise TimeoutError("超时, 已熔断")


if __name__ == "__main__":
    pass



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/4 17:37
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : setup.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
import setuptools

with open("README.md", "r") as f:
    detail_description = f.read()

requirement = [
    "APScheduler==3.6.3",
    "certifi==2019.9.11",
    "chardet==3.0.4",
    "idna==2.8",
    "pika==1.1.0",
    "pymongo==3.9.0",
    "pymssql==2.1.4",
    "PyMySQL==0.9.3",
    "python-dateutil==2.8.1",
    "python-redis-lock==3.3.1",
    "pytz==2019.3",
    "redis==3.3.11",
    "redlock-py==1.0.8",
    "requests==2.22.0",
    "six==1.13.0",
    "suds-py3==1.3.4.0",
    "tzlocal==2.0.0",
    "urllib3==1.25.7",
]


setuptools.setup(
    name="ys_service",
    version="1.0.14",
    author="7913",
    author_email="84328409@qq.com",
    description="云上host,python版",
    long_description=detail_description,
    long_description_content_type="text/markdown",
    install_requires=requirement,
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
    ]
)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/11 16:54
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : redis.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :
from redis import StrictRedis
from redis_lock import Lock
from ys_service.db_service.base_service import BaseDB

REDIS_DEFAULT_HOST = "localhost"
REDIS_DEFAULT_PORT = 6379
REDIS_DEFAULT_USERNAME = ""
REDIS_DEFAULT_PASSWORD = ""
REDIS_DEFAULT_DB = 0


class RedisService(BaseDB):

    def connect(self, host=None, port=None, password=None, db=None):
        """
        连接到redis
        :param host:
        :param port:
        :param password:
        :param db:
        :return:
        """
        _host = host if host else REDIS_DEFAULT_HOST or None
        _port = port if port else REDIS_DEFAULT_PORT or None
        _password = password if password else REDIS_DEFAULT_PASSWORD or None
        _db = db if db else REDIS_DEFAULT_DB or None
        hash_value = self.get_hash(_host, _port, _password, _db)
        if not self.is_in_map(hash_value):
            conn = YsRedis(host=_host, port=_port, password=_password, db=_db)
            self.add_to_map(hash_value, conn)

        return self.connect_map[hash_value]


class YsRedis(StrictRedis):

    def get_lock(self, lock_name, lock_ttl=None):
        """
                申请锁
                :param lock_name: lock的名字
                :param lock_ttl: 锁的生存时间, 单位为“秒”
                :return:
                """
        lock = YsLock(
            redis_client=self, name=lock_name, expire=lock_ttl
        )
        return lock


class YsLock(Lock):

    def __init__(self, redis_client, name, expire=None, id=None, auto_renewal=False, strict=True):
        super(YsLock, self).__init__(
            redis_client, name, expire=expire,
            id=id, auto_renewal=auto_renewal,
            strict=strict
        )
        self._name = "CSRedisClientLock:" + name
        self._signal = 'CSRedisClientLock-signal:' + name



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/14 11:22
# @Author  : wiken
# @Site    : 
# @File    : mq_service.py
# @Software: PyCharm
# @Desc    :
import time
from typing import Optional, Any

from ys_service.http_service._http import _HTTP
from ys_service.http_service.utils import *


class MQService(_HTTP):
    """
    本服务实现对队列消息的操作
    """
    def push_msg(self, data):
        """
        发送消息到队列
        :param data:
        :return:
        """
        url = self.base_url + "/api/MQ/PushMessage"
        res = self.request.post(url, json=data)

        return check_http_response(res)

    def pop_msg(self, data):
        """
        删除队头的一条消息并返回该条消息
        :param data:
        :return:
        """
        url = self.base_url + "/api/MQ/PopMessage"
        res = self.request.post(url, json=data)

        return check_http_response(res)

    def send_response(self, data):
        """
        向队列发送一条消息
        :param data:
        :return: info
        """
        url = self.base_url + "/api/MQ/SendResponse"
        res = self.request.post(url, json=data)
        return check_http_response(res)

    def send_request(self, data):
        """
        向指定队列发送一条,并等待回复
        :param data:
        :return: response
        """
        url = self.base_url + "/api/MQ/SendRequest"
        res = self.request.post(url, json=data)

        return check_http_response(res)

    def get_response(self, correlation_id, routing_key=None):
        """
        通过 correlationd_id 获取 send_response 的结果
        :param correlation_id:
        :param routing_key:
        :return:
        """
        param = {
            "correlationId": correlation_id,
            "routingKey": routing_key
        }

        url = self.base_url + "/api/MQ/GetResponse"
        res = self.request.get(url, params=param)

        return check_http_response(res)

    def wait_response(self, data=None, timeout=60, correlation_id=None):
        """
        发送push_msg 并循环获取数据结果，超时结果, 超时时间不能小于3s
        使用方法：
        1、传入参数data为字典类型， correlation_id = None 时， 为先push msg 再循环等待接收内容
        2、当不传data， 只传入correlation_id 的时候，只进行针对该 id 的循环接收内容操作
        :param correlation_id:
        :param data: dict
        :param timeout: 最长等待时候
        :return: {"is_success": True, "status_code": 200,"message": "", "data":data}
        """

        def wait_for_response():
            while True:
                time.sleep(1)
                time_passed = int(time.time() - start_time)
                print(time_passed)

                if time_passed >= timeout:
                    return return_msg(is_success=False, msg="获取超时")
                if time_passed % 3 == 0:
                    # 返回[] 或数据dict
                    result_dic = self.get_response(correlation_id=correlation_id)
                    if result_dic:
                        return return_msg(is_success=True, data=result_dic)

        start_time = int(time.time())

        if not correlation_id:
            correlation_id = hash(time.time())
            data["correlationId"] = correlation_id
            res = self.push_msg(data)
            if res.get("isSuccess"):
                return wait_for_response()
            else:
                return return_msg(is_success=False,
                                  msg="push msg 失败, status_code:{}".format(res.status_code))
        else:
            return wait_for_response()


class CreateMQ(object):

    @staticmethod
    def create():
        """
        创建mq_service 对象
        :return:
        """
        return MQService()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-

# @Time    : 2019/11/11 17:38
# @Author  : v5yangzai
# @Site    : https://github.com/v5yangzai
# @File    : base.py
# @project : ys_module
# @Software: PyCharm
# @Desc    :

from abc import ABC, abstractmethod


class BaseDB(ABC):

    def __init__(self):
        self.connect_map = {}

    @abstractmethod
    def connect(self, **kwargs):
        """
        创建数据库连接
        :param kwargs: 连接数据库的参数
        :return:
        """
        pass

    @staticmethod
    def get_hash(*args):
        """
        获取args参数拼接字符串的hash值
        :param s:
        :return:
        """
        object_name = __class__.__name__
        a = [str(i) for i in args]
        a.append(object_name)
        return hash("".join(a))

    def is_in_map(self, hash_value):
        """
        判断hash_value的连接是否存在
        :param hash_value: 哈希值
        :return: 存在返回True， 不存在返回False
        """
        return True if self.connect_map.get(hash_value) else False

    def add_to_map(self, hash_value, conn):
        """
        把连接添加到map对象中,
        :param hash_value:
        :param conn:
        :return:
        """
        self.connect_map[hash_value] = conn

    def close(self):
        """
        关闭连接
        :return:
        """
        for conn in self.connect_map.values():
            conn.close()
